@@ -14,6 +14,7 @@
 package main
 
 import (
+	"bytes"
 	"context"
 	"flag"
 	"fmt"
@@ -31,6 +32,7 @@ import (
 
 	"golang.org/x/oauth2"
 	"golang.org/x/oauth2/endpoints"
+	"rsc.io/qr"
 )
 
 // configByHost lists default config for several public hosts.
@@ -46,6 +48,11 @@ var configByHost = map[string]oauth2.Config{
 		ClientSecret: "2b746eea028711749c5062b9fe626fed78d03cc0",
 		Endpoint:     endpoints.GitHub,
 		Scopes:       []string{"repo", "gist", "workflow"}},
+	"gist.github.com": {
+		ClientID: "b895675a4e2cf54d5c6c",
+		ClientSecret: "2b746eea028711749c5062b9fe626fed78d03cc0",
+		Endpoint:     endpoints.GitHub,
+		Scopes:       []string{"repo", "gist", "workflow"}},
 	// https://gitlab.com/oauth/applications/232663
 	"gitlab.com": {
 		ClientID: "10bfbbf46e5b760b55ce772a262d7a0205eacc417816eb84d37d0fb02c89bb97",
@@ -133,9 +140,7 @@ func parse(input string) map[string]string {
 	lines := strings.Split(string(input), "\n")
 	pairs := map[string]string{}
 	for _, line := range lines {
-		parts := strings.SplitN(line, "=", 2)
-		if len(parts) >= 2 {
-			key, value := parts[0], parts[1]
+		if key, value, ok := strings.Cut(line, "="); ok {
 			_, exists := pairs[key]
 			if strings.HasSuffix(key, "[]") && exists {
 				pairs[key] += "\n" + value
@@ -301,9 +306,15 @@ func main() {
 		}
 
 		var authURLSuffix string
-		if looksLikeGitHub && pairs["username"] != "" && pairs["username"] != "oauth2" {
-			// https://docs.github.com/en/apps/oauth-apps/building-oauth-apps/authorizing-oauth-apps#1-request-a-users-github-identity
-			authURLSuffix = fmt.Sprintf("&login=%s", pairs["username"])
+		if pairs["username"] != "" && pairs["username"] != "oauth2" {
+			if looksLikeGitHub {
+				// https://docs.github.com/en/apps/oauth-apps/building-oauth-apps/authorizing-oauth-apps#1-request-a-users-github-identity
+				authURLSuffix = fmt.Sprintf("&login=%s", pairs["username"])
+			}
+			if strings.HasSuffix(host, ".googlesource.com") {
+				// https://developers.google.com/identity/protocols/oauth2/web-server#creatingclient
+				authURLSuffix = fmt.Sprintf("&login_hint=%s", pairs["username"])
+			}
 		}
 
 		if token == nil {
@@ -401,6 +412,7 @@ var template string = `<!DOCTYPE html>
 <html lang="en">
 <head>
 	<title>Git authentication</title>
+	<meta name="color-scheme" content="light dark" />
 </head>
 <body>
 <p>Success. You may close this page and return to Git.</p>
@@ -487,15 +499,54 @@ func getToken(ctx context.Context, c oauth2.Config, authURLSuffix string) (*oaut
 func getDeviceToken(ctx context.Context, c oauth2.Config) (*oauth2.Token, error) {
 	deviceAuth, err := c.DeviceAuth(ctx)
 	if err != nil {
-		log.Fatalln(err)
+		return nil, err
 	}
 	if verbose {
 		fmt.Fprintf(os.Stderr, "%+v\n", deviceAuth)
 	}
-	fmt.Fprintf(os.Stderr, "Please enter code %s at %s\n", deviceAuth.UserCode, deviceAuth.VerificationURI)
+	if deviceAuth.VerificationURIComplete != "" {
+		fmt.Fprintf(os.Stderr, "Please scan the QR code or enter code %s at %s\n", deviceAuth.UserCode, deviceAuth.VerificationURI)
+		fmt.Fprintln(os.Stderr)
+		writeQRCode(os.Stderr, deviceAuth.VerificationURIComplete)
+	} else {
+		fmt.Fprintf(os.Stderr, "Please enter code %s at %s\n", deviceAuth.UserCode, deviceAuth.VerificationURI)
+	}
 	return c.DeviceAccessToken(ctx, deviceAuth)
 }
 
+func writeQRCode(w io.Writer, data string) error {
+	// use low redundancy to generate small QR codes for terminal output.
+	// we assume the user is sitting in front of their screen so image quality shouldn't be an issue
+	code, err := qr.Encode(data, qr.L)
+	if err != nil {
+		return err
+	}
+
+	var (
+		black     = "\033[40m  \033[0m"
+		white     = "\033[107m  \033[0m"
+		whiteLine = strings.Repeat(white, code.Size+2) + "\n"
+		buf       bytes.Buffer
+	)
+	buf.WriteString(whiteLine)
+	for y := range code.Size {
+		buf.WriteString(white)
+		for x := range code.Size {
+			if code.Black(x, y) {
+				buf.WriteString(black)
+			} else {
+				buf.WriteString(white)
+			}
+		}
+		buf.WriteString(white)
+		buf.WriteRune('\n')
+	}
+	buf.WriteString(whiteLine)
+
+	_, err = buf.WriteTo(w)
+	return err
+}
+
 func replaceHost(e oauth2.Endpoint, host string) oauth2.Endpoint {
 	e.AuthURL = replaceHostInURL(e.AuthURL, host)
 	e.TokenURL = replaceHostInURL(e.TokenURL, host)
